//////////////////////////////////////////////////
//////////////////////////////////////////////////
// This source code is Copyright (2011) of      //
//  Jeppe R. Andersen and Jennifer M. Smillie   //
// and is distributed under the                 //
// Gnu Public License version 2                 //
// http://www.gnu.org/licenses/gpl-2.0.html     //
// You are allowed to distribute and alter the  //
// source under the conditions of the GPLv2     //
// as long as this copyright notice             //
// is unaltered and distributed with the source //
// Any use should comply with the               //
//             MCNET GUIDELINES                 //
//    for Event Generator Authors and Users     //
// as distributed with this source code         //
//////////////////////////////////////////////////
//////////////////////////////////////////////////
#ifndef _MULTIJET_H_INCLUDED_
#define _MULTIJET_H_INCLUDED_
#include <math.h>
#include <complex>
#include <list>
#include <CLHEP/Vector/LorentzVector.h>
#include <CLHEP/Random/Randomize.h>
#include <CLHEP/Random/RanluxEngine.h>
#include "fastjet/PseudoJet.hh"
#include "fastjet/ClusterSequence.hh"
#include "mstwpdf.h"
#include "GeneratorFlags.h"

typedef std::complex<double> COM;
typedef double FREAL;

#define printoutput 0
using namespace std;
using namespace CLHEP;

struct particle {
  int PID;
  int COL1;
  int COL2;
  int MOT1;
  int MOT2;
  int ISTUP;
  CLHEP::HepLorentzVector p;
};

class CMultijet
{
private:
  // Generator flags:
  SGeneratorFlags GeneratorFlags;
  
  double wt; // The weight for the matching correction
/*   double sqrts; */
  //  double pt1,pt2;

  std::vector<CLHEP::HepLorentzVector> partonmomenta;
  // For making a nice plot
  std::vector<CLHEP::HepLorentzVector> origpartons;
  std::vector<CLHEP::HepLorentzVector> fjjets;
  std::vector<CLHEP::HepLorentzVector> finaljets;
  HepLorentzVector pa,pb,ti;
  std::vector<particle> LHparticles;
  ofstream LHoutput;
  long xsecpos;
  static int const GLUEINT=20; // the max number of gluons -1
  //  static int const GLUEINT=2; // the max number of gluons -1
  //  static int const GLUEINT=12; // the max number of gluons -1
  //  static double const RAPINT=10.0; // the maxlength of the rapidity interval
  static double const beta0=11.-2.*5/3.;
  double lambda; // the phase space slicing regularisation scale
  
  int aptype,bptype; // a is backward, b forward
  double xa,xb; // xa-, xb+
  double matchingweight,matchingweight2;
  double ymax; // the maximum rapidity generated for any particle
  class c_mstwpdf * parton;
  void AddParton(double p[4]);
  double omega0(double q2, double lambda2);
  double beta0nc;
  FREAL pmg3[7][4];
  double MEt2(std::vector<CLHEP::HepLorentzVector> partons);
  double MEt2(std::vector<CLHEP::HepLorentzVector> partons, CLHEP::HepLorentzVector pa, CLHEP::HepLorentzVector pb,bool checkmomenta,bool virt, fastjet::ClusterSequence * ev);
  double UberME2(std::vector<CLHEP::HepLorentzVector> partons, CLHEP::HepLorentzVector pa, CLHEP::HepLorentzVector pb, bool checkcl, bool virt);
  double MG(std::vector<CLHEP::HepLorentzVector> partons, CLHEP::HepLorentzVector pa, CLHEP::HepLorentzVector pb);
  double MG3(std::vector<CLHEP::HepLorentzVector> partons, CLHEP::HepLorentzVector pa, CLHEP::HepLorentzVector pb);
  double MG4(std::vector<CLHEP::HepLorentzVector> partons, CLHEP::HepLorentzVector pa, CLHEP::HepLorentzVector pb);
  double VirtualCorrections(std::vector<CLHEP::HepLorentzVector> partonmomenta, CLHEP::HepLorentzVector pa);
  double GeneratePHSPpoint();
  double Generate2jphsppoint();
  double Generate3jphsppoint();
  double Generate4jphsppoint();
  double Generate2JNonFKLflavourconf_and_eval(double p[4][4]);
  double Generate3JNonFKLflavourconf_and_eval(double p[5][4]);
  double Generate4JNonFKLflavourconf_and_eval(double p[6][4]);
  double pdfpt(int beam, double a, double q,int t);
  double Halphas(double q);
  double GenerateFKLIncomingParticles(double xa,double ufa,double xb,double ufb);
  double GenerateIncomingParticles(double xa,double ufa,double xb,double ufb);
  void SetupLHInformation(std::vector<particle> *LHparticles,std::vector<CLHEP::HepLorentzVector> *partonmomenta,CLHEP::HepLorentzVector *pa,CLHEP::HepLorentzVector *pb);

  int status; // the return status of an event
  int processtype; // the process type of an event - matching, 2,3,4 jets
  static const int nprocesstype=4; // resummed(&matched), 2jet, 3jet, 4jet
  double MHEJsave;
  // Scale setting parameters
  //  double PTMAX;  // Pt of hardest jet in the event
  double renscale;  // scale, either fixed or PTMAX
  // jets:
  std::vector<fastjet::PseudoJet> jets;

public:
  // constructor & destructor to be defined later
  virtual ~CMultijet(void);
  CMultijet(void);
  CMultijet(string filename);
  CMultijet(SGeneratorFlags GeneratorFlagsa);

  double pdfa,pdfb,pdfda,pdfdb,pdfuxa,pdfuxb,pdfdxa,pdfdxb,pdfga,pdfba,pdfgb,pdfsa,pdfsb,pdfsxa,pdfsxb,pdfca,pdfcb,pdfua,pdfub,pdfbb,pdfta,pdftb;
  SGeneratorFlags flags;
  double getcmsenergy();
  double weight();  // resummed (and matched ?)
  double weight1(); // 2-jet
  double weight2(); // 3-jet
  double weight3(); // 4-jet
  double Event();
  int getnp();
  HepLorentzVector getpartvn(int n);
  int getMstatus(void);
  HepRandom ran;
  int getstatus();
  int getprocesstype();
  void setstatus(int s);
  double GenerateEvent();
  void LHSaveEvent(void);
  double xsec[nprocesstype+1][2];
  double maxweight[nprocesstype+1];
  double NLLCorrect(double newpar);
  std::vector<particle> GetLHParticles();
  std::vector<particle> GetRedLHParticles();
  double SplitSubtract(std::vector<CLHEP::HepLorentzVector>,double del[3]);
  bool AwayFromCol(std::vector<CLHEP::HepLorentzVector>);
  double FindPartonr(std::vector<CLHEP::HepLorentzVector> testemissions);
  vector<CLHEP::HepLorentzVector> getorigpartons(void);
  vector<CLHEP::HepLorentzVector> getfinaljets(void);
  vector<CLHEP::HepLorentzVector> getfjjets(void);
};
#endif

